@@ -67,7 +67,8 @@ typedef enum
   WNCK_WINDOW_ACTION_MAXIMIZE                = 1 << 14,
   WNCK_WINDOW_ACTION_UNMAXIMIZE              = 1 << 15,
   WNCK_WINDOW_ACTION_FULLSCREEN              = 1 << 16,
-  WNCK_WINDOW_ACTION_ABOVE                   = 1 << 17
+  WNCK_WINDOW_ACTION_ABOVE                   = 1 << 17,
+  WNCK_WINDOW_ACTION_BELOW                   = 1 << 18
 } WnckWindowActions;
 
 typedef enum
@@ -83,6 +84,29 @@ typedef enum
   WNCK_WINDOW_SPLASHSCREEN  /* splash screen */
 } WnckWindowType;
 
+typedef enum
+{
+  WNCK_WINDOW_GRAVITY_CURRENT   = 0,
+  WNCK_WINDOW_GRAVITY_NORTHWEST = 1,
+  WNCK_WINDOW_GRAVITY_NORTH     = 2,
+  WNCK_WINDOW_GRAVITY_NORTHEAST = 3,
+  WNCK_WINDOW_GRAVITY_WEST      = 4,
+  WNCK_WINDOW_GRAVITY_CENTER    = 5,
+  WNCK_WINDOW_GRAVITY_EAST      = 6,
+  WNCK_WINDOW_GRAVITY_SOUTHWEST = 7,
+  WNCK_WINDOW_GRAVITY_SOUTH     = 8,
+  WNCK_WINDOW_GRAVITY_SOUTHEAST = 9,
+  WNCK_WINDOW_GRAVITY_STATIC    = 10
+} WnckWindowGravity;
+
+typedef enum
+{
+  WNCK_WINDOW_CHANGE_X      = 1 << 0,
+  WNCK_WINDOW_CHANGE_Y      = 1 << 1,
+  WNCK_WINDOW_CHANGE_WIDTH  = 1 << 2,
+  WNCK_WINDOW_CHANGE_HEIGHT = 1 << 3
+} WnckWindowMoveResizeMask;
+
 #define WNCK_TYPE_WINDOW              (wnck_window_get_type ())
 #define WNCK_WINDOW(object)           (G_TYPE_CHECK_INSTANCE_CAST ((object), WNCK_TYPE_WINDOW, WnckWindow))
 #define WNCK_WINDOW_CLASS(klass)      (G_TYPE_CHECK_CLASS_CAST ((klass), WNCK_TYPE_WINDOW, WnckWindowClass))
@@ -151,6 +175,8 @@ const char* wnck_window_get_session_id        (WnckWindow *window);
 const char* wnck_window_get_session_id_utf8   (WnckWindow *window);
 int         wnck_window_get_pid               (WnckWindow *window);
 gint        wnck_window_get_sort_order        (WnckWindow *window);
+void        wnck_window_set_sort_order        (WnckWindow *window, 
+						gint order);
 
 WnckWindowType wnck_window_get_window_type    (WnckWindow *window);
 void           wnck_window_set_window_type    (WnckWindow *window,
@@ -232,6 +258,13 @@ void wnck_window_get_geometry (WnckWindow *window,
                                int        *yp,
                                int        *widthp,
                                int        *heightp);
+void wnck_window_set_geometry (WnckWindow               *window,
+                               WnckWindowGravity         gravity,
+                               WnckWindowMoveResizeMask  geometry_mask,
+                               int                       x,
+                               int                       y,
+                               int                       width,
+                               int                       height);
 
 gboolean wnck_window_is_visible_on_workspace (WnckWindow    *window,
                                               WnckWorkspace *workspace);
